﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Diagnostics;
using System.Threading;
using System.Drawing;
using System.Text;
using Styx;
using Styx.Common;
using Styx.Common.Helpers;
using Styx.CommonBot;
using Styx.TreeSharp;
using Styx.Helpers;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.Pathing;
using Styx.Plugins;


namespace GroupGreet
{
    public class GroupGreeter : HBPlugin
    {
        public override string Name { get { return "GroupGreeter"; } }
        public override string Author { get { return "wownerds, aash"; } }
        public override Version Version { get { return new Version(0, 0, 3, 4); } }
        public override string ButtonText { get { return "Config"; } }
        private static LocalPlayer Me { get { return StyxWoW.Me; } }

        private const string DefaultChatType = "INSTANCE_CHAT";

        public override bool WantButton { get { return true; } }

        private GroupGreeterCFG ConfigForm { get; set; }
        public override void OnButtonPress()
        {
            if (ConfigForm == null)
                ConfigForm = new GroupGreeterCFG();
            ConfigForm.ShowDialog();
            GreetMessages.Clear();
        }

        private readonly WaitTimer _pulseTimer = WaitTimer.FiveSeconds;

        private bool _isInGroup;

        private readonly List<string> _greetMessages = new List<string>();
        public List<string> GreetMessages
        {
            get
            {
                if (_greetMessages.Any())
                    return _greetMessages;
                if (GroupGreeterSettings.Instance.g1_enable)
                    _greetMessages.Add(GroupGreeterSettings.Instance.greeting1);
                if (GroupGreeterSettings.Instance.g2_enable)
                    _greetMessages.Add(GroupGreeterSettings.Instance.greeting2);
                if (GroupGreeterSettings.Instance.g3_enable)
                    _greetMessages.Add(GroupGreeterSettings.Instance.greeting3);
                if (GroupGreeterSettings.Instance.g4_enable)
                    _greetMessages.Add(GroupGreeterSettings.Instance.greeting4);
                if (GroupGreeterSettings.Instance.g5_enable)
                    _greetMessages.Add(GroupGreeterSettings.Instance.greeting5);
                return _greetMessages;
            }
        }

        public override void OnEnable()
        {
            base.OnEnable();
            GroupGreeterSettings.Instance.Load();
            Log("ENABLED!");
            _isInGroup = false;
            Pulse();
        }

        public override void OnDisable()
        {
            base.OnDisable();
            _isInGroup = false;
            Log("DISABLED!");
        }

        public override void Pulse()
        {
            // throttle pulses by X second interval
            if (!_pulseTimer.IsFinished)
                return;
            _pulseTimer.Reset();

            // check current group status
            var isCurrentlyInGroup = Me.GroupInfo.IsInParty || Me.GroupInfo.IsInLfgParty || Me.GroupInfo.IsInRaid;

            Log("isCurrentlyInGroup: {0}, _isInGroup: {0}", isCurrentlyInGroup, _isInGroup);

            if (isCurrentlyInGroup)
                TankMark();

            // group state did not changed, so dont care
            if (isCurrentlyInGroup == _isInGroup)
                return;

            // greet only if I was invited to group
            if (!_isInGroup)
            {
                Log("You joined a new group!");
                Greet();
            }

            _isInGroup = isCurrentlyInGroup;
        }

        private readonly Random _rnd = new Random();
        
        private void Greet()
        {
            int idx = _rnd.Next(GreetMessages.Count);
            var message = GreetMessages[idx];
            Log("sending greet message to the group: \'{0}\'", message);
            SendChatMessage(message);
        }

        private void LogDiag(string format, params object[] args)
        {
            var message = string.Format("[{0}][DIAG]: {1}", Name, string.Format(format, args));
            Logging.WriteDiagnostic(message);
        }
        
        private void Log(string format, params object[] args)
        {
            var message = string.Format("[{0}]: {1}", Name, string.Format(format, args));
            Logging.Write(message);
        }
        
        private void SendChatMessage(string message)
        {
            var _chatType = DefaultChatType;
            if (Me.GroupInfo.IsInParty)
                _chatType = "PARTY";
            if (Me.GroupInfo.IsInLfgParty)
                _chatType = "INSTANCE_CHAT";
            if (Me.GroupInfo.IsInRaid)
                _chatType = "RAID";
            var lua = string.Format("SendChatMessage('{0}', '{1}');", message, _chatType);
            Lua.DoString(lua);
            LogDiag("{0}", lua);
        }

        private void TankMark()
        {
            if ((
                Me.GroupInfo.IsInParty
                || Me.GroupInfo.IsInLfgParty
                || Me.GroupInfo.IsInRaid)
                && Me.CurrentTarget != null
                && Me.CurrentTarget.IsBoss
                && Me.CurrentTarget.GetMark() != RaidTargetMarker.Skull
                && GroupGreeterSettings.Instance.tank_marking
                && Me.IsInInstance
                && Me.CurrentTarget.IsHostile)
            {
                Me.CurrentTarget.Mark(RaidTargetMarker.Skull);
                Log("Marked {0} with Skull Raidmark.", Me.CurrentTarget.Name);
            }
        }

    }
}
